@@ -49,6 +49,15 @@ app: Flask = Flask(
 )
 
 
+# noinspection HttpUrlsUsage
+def redirect_http_server(host: str, port: int) -> WSGIServer:
+    app_redirect: Flask = Flask("Redirect HTTP")
+    app_redirect.before_request(
+        lambda: redirect(request.url.replace("http://", "https://", 1), 301) if not request.is_secure else None)
+    app_redirect_server: WSGIServer = WSGIServer((host, port), app_redirect)
+    return app_redirect_server
+
+
 def button(href: str, text: str) -> str:
     return f'<button onclick="window.location = \'{href}\'">{text}</button>'
 
@@ -79,6 +88,8 @@ def response_minify(response: Response):
     return response
 
 
+@app.route("/apple-touch-icon-precomposed.png")
+@app.route("/apple-touch-icon.png")
 @app.route("/favicon.ico")
 def serve_favicon():
     return serve_static_file("favicon.ico")
@@ -368,14 +379,31 @@ def serve_static_file(filename: Union[str, PathLike]):
     return send_file(filepath, attachment_filename=filepath.name) if filepath.is_file() else abort(404)
 
 
-def server(database_path: Union[str, PathLike], host: str = "0.0.0.0", port: int = 8080):
+# noinspection HttpUrlsUsage
+def server(database_path: Union[str, PathLike], host: str = "0.0.0.0", port: int = None,
+           ssl_cert: Union[str, PathLike] = None, ssl_key: Union[str, PathLike] = None, redirect_http: bool = False):
     app.config["db_path"] = Path(database_path).resolve()
-    app_server: WSGIServer = WSGIServer((host, port), app)
-    print(f"Using database {app.config['db_path']}")
-    print(f"Serving app on http://{app_server.server_host}:{app_server.server_port}")
+    print("Using database", app.config['db_path'])
+    if ssl := ssl_cert and ssl_key:
+        port = port or 443
+        ssl_cert, ssl_key = Path(ssl_cert), Path(ssl_key)
+        print("Using SSL certificate", ssl_cert)
+        print("Using SSL key", ssl_key)
+    else:
+        port = port or 80
+        ssl_cert = ssl_key = None
+    app_server: WSGIServer = WSGIServer((host, port), app, certfile=ssl_cert, keyfile=ssl_key)
+    redirect_server: Optional[WSGIServer] = redirect_http_server(host, 80) if redirect_http and ssl and port != 80 else None
+    print(f"Serving app on {'https' if ssl else 'http'}://{app_server.server_host}:{app_server.server_port}")
+    if redirect_server:
+        print(f"Redirecting from http://{redirect_server.server_host}:{redirect_server.server_port}")
     try:
+        if redirect_server:
+            redirect_server.start()
         app_server.serve_forever()
     except KeyboardInterrupt:
         pass
     finally:
+        if redirect_server:
+            redirect_server.stop()
         app_server.stop()
